==> goblin-2.4a3.tgz: include/goblin.2.4/bal2bal.h <==

/*
**  This file forms part of the GOBLIN C++ Class Library.
**
**  Written by Christian Fremuth-Paeger, May 2000.
**  
**  Copying, compiling, distribution and modification
**  of this source code is permitted only in accordance
**  with the GOBLIN general licence information.
*/             


#if !defined(_BAL2BAL_H_)

#define _BAL2BAL_H_

#include <absbal.h>


class bal2bal : public abstractBalancedFNW
{                                        
protected:

    abstractBalancedFNW & G; // Original network

    TNode           n0;      // Number of nodes in the original network
    TArc            m0;      // Number of arcs in the original network
    TNode           k2;      // Number of odd cycles
    TNode           k1;      // k1 == k2/2

    TNode           s1;      // Source node, s1 == Source()
    TNode           t1;      // Sink node, t1 == Target()
    TNode           s2;      // Additional source node
    TNode           t2;      // Additional target node
    TArc            ret1;    // Return arc with end nodes t1 and s1
    TArc            ret2;    // Return arc with end nodes t2 and s2
    TArc            art1;    // Artifical arc with end nodes s1 and t2
    TArc            art2;     // Artifical arc with end nodes s2 and t1

    TFloat *        flow;      // Flow values on the artifical arcs
    TArc *          artifical;
        // artifical[v] connects the original node v to s2 or t2.
        // Only partially defined, 2*k2 arcs in total.
    TNode *         repr;      // Canonical elements of the odd cycles
    
    char            symm;      // Is flow balanced?
    
public:             

    bal2bal(abstractBalancedFNW &GC) throw();
    ~bal2bal() throw();

    unsigned long   Size() throw();     
    unsigned long   Allocated() throw();     
    
    char            Perfect() throw();

    void            Relax() throw();
    void            Symmetrize() throw();

    TNode           Source() throw() {return s1;};
    TNode           Target() throw(){return t1;};     

    TNode           StartNode(TArc a) throw(ERRange);
    TNode           EndNode(TArc a) throw(ERRange);
   
    TArc            First(TNode v) throw(ERRange);
    TArc            Right(TArc a,TNode u) throw(ERRange,ERRejected);

    TCap            Demand(TNode v) throw(ERRange);
    TCap            LCap(TArc a) throw(ERRange);
    char            CLCap() throw() {return G.CLCap() && (G.MaxLCap()==0);};
    TCap            UCap(TArc a) throw(ERRange);
    char            CUCap() throw() {return 0;};
    TFloat          Length(TArc a) throw(ERRange);
    char            CLength() throw()
                        {return G.CLength() && (G.MaxLength()==0);};
    char            Blocking(TArc a) throw() {return a&1;};

    TFloat          C(TNode v,char i) throw(ERRange);
    TFloat          CMax(char i) throw(ERRange);
    char            Dim() throw() {return G.Dim();};
    char            HiddenNode(TNode v) throw(ERRange);

    TFloat          Flow(TArc a) throw(ERRange);
    TFloat          BalFlow(TArc a) throw(ERRange);
    void            Push(TArc a,TFloat Lambda) throw(ERRange,ERRejected);
    void            BalPush(TArc a,TFloat Lambda) throw(ERRange,ERRejected);
    
    TFloat          Sub(TArc a) throw(ERRange) {return BalFlow(a);};
    void            AddArc(TArc a,TFloat Lambda) throw() {};
    void            OmitArc(TArc a,TFloat Lambda) throw() {};
    
    void Write(char * fileName) throw(ERFile)
        {WriteBalancedFNW(fileName,OPT_INCIDENCES);};

};                             


#endif

==> goblin-2.6.4.tgz: include/goblin.2.6/bal2bal.h <==

/*
**  This file forms part of the GOBLIN C++ Class Library.
**
**  Written by Christian Fremuth-Paeger, May 2000.
**  
**  Copying, compiling, distribution and modification
**  of this source code is permitted only in accordance
**  with the GOBLIN general licence information.
*/             


#if !defined(_BAL2BAL_H_)

#define _BAL2BAL_H_

#include "absbal.h"


class bal2bal : public abstractBalancedFNW
{                                        
protected:

    abstractBalancedFNW & G; // Original network

    TNode           n0;      // Number of nodes in the original network
    TArc            m0;      // Number of arcs in the original network
    TNode           k2;      // Number of odd cycles
    TNode           k1;      // k1 == k2/2

    TNode           s1;      // Source node, s1 == Source()
    TNode           t1;      // Sink node, t1 == Target()
    TNode           s2;      // Additional source node
    TNode           t2;      // Additional target node
    TArc            ret1;    // Return arc with end nodes t1 and s1
    TArc            ret2;    // Return arc with end nodes t2 and s2
    TArc            art1;    // Artifical arc with end nodes s1 and t2
    TArc            art2;     // Artifical arc with end nodes s2 and t1

    TFloat*         flow;      // Flow values on the artifical arcs
    TArc*           artifical;
        // artifical[v] connects the original node v to s2 or t2.
        // Only partially defined, 2*k2 arcs in total.
    TNode *         repr;      // Canonical elements of the odd cycles
    
    bool            symm;      // Is flow balanced?
    
public:             

    bal2bal(abstractBalancedFNW &GC) throw();
    ~bal2bal() throw();

    unsigned long   Size() throw();     
    unsigned long   Allocated() throw();     
    
    bool            Perfect() throw();

    void            Relax() throw();
    void            Symmetrize() throw();

    TNode           Source() throw() {return s1;};
    TNode           Target() throw(){return t1;};     

    TNode           StartNode(TArc a) throw(ERRange);
    TNode           EndNode(TArc a) throw(ERRange);
   
    TArc            First(TNode v) throw(ERRange);
    TArc            Right(TArc a,TNode u) throw(ERRange,ERRejected);

    TCap            Demand(TNode v) throw(ERRange);
    TCap            LCap(TArc a) throw(ERRange);
    bool            CLCap() throw() {return G.CLCap() && (G.MaxLCap()==0);};
    TCap            UCap(TArc a) throw(ERRange);
    bool            CUCap() throw() {return 0;};
    TFloat          Length(TArc a) throw(ERRange);
    bool            CLength() throw()
                        {return G.CLength() && (G.MaxLength()==0);};
    bool            Blocking(TArc a) throw() {return a&1;};

    TFloat          C(TNode v,char i) throw(ERRange);
    TFloat          CMax(char i) throw(ERRange);
    char            Dim() throw() {return G.Dim();};
    bool            HiddenNode(TNode v) throw(ERRange);

    TFloat          Flow(TArc a) throw(ERRange);
    TFloat          BalFlow(TArc a) throw(ERRange);
    void            Push(TArc a,TFloat Lambda) throw(ERRange,ERRejected);
    void            BalPush(TArc a,TFloat Lambda) throw(ERRange,ERRejected);
    
    TFloat          Sub(TArc a) throw(ERRange) {return BalFlow(a);};
    void            AddArc(TArc a,TFloat Lambda) throw() {};
    void            OmitArc(TArc a,TFloat Lambda) throw() {};
    
    void            Write(char* fileName) throw(ERFile)
                        {WriteBalancedFNW(fileName,WRITE_INCIDENCES);};

};                             


#endif
